import std.string;

import lua;
import lualib;
import lauxlib;

import log;
import charactor;
import game;
import colors;

private:

extern (C) {
	int luaPanic(lua_State lua) {
		Log.error("lua error" ~ std.string.toString(lua_tostring(lua, 0)));
		return 0;
	}

	Charactor nowBattery;
	Command nowCommand;

	void throwError(lua_State lua, char[] msg) {
		Log.error(msg);
	}

	float getDouble(lua_State lua, int i) {
		if (!lua_isnumber(lua, i)) {
			throwError(lua, "incorrect argument");
		}
		return lua_tonumber(lua, i);
	}

	int getInt(lua_State lua, int i) {
		return cast(int)(getDouble(lua, i));
	}

	char[] getString(lua_State lua, int i) {
		if (!lua_isstring(lua, i)) {
			throwError(lua, "incorrect argument");
		}
		return std.string.toString(lua_tostring(lua, i));
	}

	int fireAbs(lua_State lua) {
		int n = lua_gettop(lua);
		if (n < 3 || n > 5) {
			throwError(lua,
					   "fire must have 3 or 4 or 5 arguments, but comes " ~
					   std.string.toString(n) ~ " arguments");
		}

		double d = getDouble(lua, 1);
		double s = getDouble(lua, 2);
		int type = getInt(lua, 3);
		if (n == 3) {
			nowCommand.addShotAbs(d, s, type);
		}
		else if (n == 4) {
			char[] func = getString(lua, 4);
			nowCommand.addBulletAbs(d, s, type, lua, func);
		}
		else if (n == 5) {
			char[] func = getString(lua, 4);
			int id = getInt(lua, 5);
			nowCommand.addBulletAbs(d, s, type, lua, func);
		}
		return 0;
	}
	int fireAim(lua_State lua) {
		int n = lua_gettop(lua);
		if (n < 3 || n > 5) {
			throwError(lua,
					   "fire must have 3 or 4 or 5 arguments, but comes " ~
					   std.string.toString(n) ~ " arguments");
		}

		double d = getDouble(lua, 1);
		double s = getDouble(lua, 2);
		int type = getInt(lua, 3);
		if (n == 3) {
			nowCommand.addShotAim(d, s, type);
		}
		else if (n == 4) {
			char[] func = getString(lua, 4);
			nowCommand.addBulletAim(d, s, type, lua, func);
		}
		else if (n == 5) {
			char[] func = getString(lua, 4);
			int id = getInt(lua, 5);
			nowCommand.addBulletAim(d, s, type, lua, func);
		}
		return 0;
	}

	int changeSpeed(lua_State lua) {
		int n = lua_gettop(lua);
		if (n != 1) {
			throwError(lua, "changeSpeed must have 1 arguments");
		}
		nowCommand.doChangeSpeed(getDouble(lua, 1));
		return 0;
	}
	int changeDirectionAbs(lua_State lua) {
		int n = lua_gettop(lua);
		if (n != 1) {
			throwError(lua, "changeDirection must have 1 arguments");
		}
		nowCommand.doChangeDirectionAbs(getDouble(lua, 1));
		return 0;
	}
	int changeDirectionAim(lua_State lua) {
		int n = lua_gettop(lua);
		if (n != 1) {
			throwError(lua, "changeDirection must have 1 arguments");
		}
		nowCommand.doChangeDirectionAim(getDouble(lua, 1));
		return 0;
	}
	int vanish(lua_State lua) {
		nowCommand.doVanish();
		return 0;
	}
	int wait(lua_State lua) {
		int n = lua_gettop(lua);
		if (n != 1) {
			throwError(lua, "wait must have 1 arguments");
		}
		nowCommand.doWait(getInt(lua, 1));
		return 0;
	}
	int getRank(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)0.5);
		return 1;
	}
	int getTurn(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)nowCommand.doGetTurn());
		return 1;
	}
	int getX(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)nowBattery.x());
		return 1;
	}
	int getY(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)nowBattery.y());
		return 1;
	}
	int getSpeedX(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)nowBattery.sx());
		return 1;
	}
	int getSpeedY(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)nowBattery.sy());
		return 1;
	}
	int getMaxX(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)Cave.obj().screen().w);
		return 1;
	}
	int getMaxY(lua_State lua) {
		lua_pushnumber(lua, cast(lua_Number)Cave.obj().screen().h);
		return 1;
	}

}

public:

class LuaFile {
public:
	this(char[] file) {
		Log.info(file ~ ": lua file open");

		lua = lua_open();
		lua_atpanic(lua, cast(lua_CFunction)&luaPanic);

		lua_baselibopen(lua);
		lua_mathlibopen(lua);

		int ret = luaL_loadfile(lua, std.string.toStringz(file));

		if (ret != 0) {
			if (ret == LUA_ERRSYNTAX)
				Log.error(file ~ ": syntax error");
			else if (ret == LUA_ERRMEM)
				Log.error(file ~ ": memory error");
			else
				Log.error(file ~ ": some error");
		}

		lua_pcall(lua, 0, LUA_MULTRET, 0);

		lua_register(lua, "fireAim", &fireAim);
		lua_register(lua, "fireAbs", &fireAbs);
		lua_register(lua, "vanish", &vanish);
		lua_register(lua, "wait", &wait);
		lua_register(lua, "changeSpeed", &changeSpeed);
		lua_register(lua, "changeDirectionAim", &changeDirectionAim);
		lua_register(lua, "changeDirectionAbs", &changeDirectionAbs);
		lua_register(lua, "getRank", &getRank);
		lua_register(lua, "getTurn", &getTurn);
		lua_register(lua, "getX", &getX);
		lua_register(lua, "getY", &getY);
		lua_register(lua, "getSpeedX", &getSpeedX);
		lua_register(lua, "getSpeedY", &getSpeedY);
		lua_register(lua, "getMaxX", &getMaxX);
		lua_register(lua, "getMaxY", &getMaxY);
	}
	lua_State lua;
}

// ȤꤢŨ lua 椹ǡ
class Command {
public:
	this(lua_State lua, char[] fn, Game game, Charactor battery, int color) {
		game_ = game;
		battery_ = battery;
		func_ = fn;
		lua_ = lua;
		color_ = color;
	}

	this(LuaFile file, char[] fn, char[] fnsub,
         Game game, Charactor battery, int color) {
		game_ = game;
		battery_ = battery;
		func_ = fn;
		color_ = color;
		lua_ = file.lua;

		lua_getglobal(lua_, std.string.toStringz(func_));
		if (!lua_isfunction(lua_, -1)) {
			func_ = fnsub;
		}

		lua_getglobal(lua_, std.string.toStringz(func_));
		if (!lua_isfunction(lua_, -1)) {
			throwError(lua_, fn ~ "&" ~ fnsub ~ ": function not found");
		}
	}
/*
	this(char[] file, Game game, Charactor battery, int color) {
		this(file, "top", game, battery, color);
	}
*/
	void init() {
	}

	void run() {
		nowCommand = this;
		nowBattery = battery_;

		lua_getglobal(lua_, std.string.toStringz(func_));

		if (!lua_isfunction(lua_, -1)) {
			Log.warn(func_ ~ ": function not found");
			return;
			//throwError(lua_, func_ ~ ": function not found");
		}

        int ret = 0;
//        if (id_ == -1) {
		ret = lua_pcall(lua_, 0, 0, 0);
//        }
//        else {
//			lua_pushnumber(lua_, id_);
//			ret = lua_pcall(lua_, 1, 0, 0);
//        }

        if (ret) {
			throwError(lua_, func_ ~ ": function errored");
        }
	}

	void addShotAim(float d, float s, int type) {
		game_.addShotAim(battery_.x(), battery_.y(), d, s, color_, type);
	}
	void addBulletAim(float d, float s, int type, lua_State lua, char[] fn) {
		game_.addBulletAim(battery_.x(), battery_.y(),
						   d, s, color_, type, lua, fn);
	}

	void addShotAbs(float a0, float s, int type) {
		float a = convRelAngle(a0);
		game_.addShotAbs(battery_.x(), battery_.y(), a, s, color_, type);
	}
	void addBulletAbs(float a0, float s, int type, lua_State lua, char[] fn) {
		float a = convRelAngle(a0);
		game_.addBulletAbs(battery_.x(), battery_.y(),
						   a, s, color_, type, lua, fn);
	}

	int doGetTurn() {
		return battery_.turn();
	}
	void doVanish() {
		game_.vanishBullet(battery_, color_);
	}
	void doWait(int t) {
		battery_.wait(t);
	}

	void doChangeSpeed(float s) {
		if (s < 0.5) return;

		float px = game_.player(color_).x();
		float py = game_.player(color_).y();
		float tx = game_.player(Colors.opposite(color_)).x();
		float ty = game_.player(Colors.opposite(color_)).y();
		float x = battery_.x();
		float y = battery_.y();
		float nearFact =
			((tx-px)*(tx-px)+(ty-py)*(ty-py))/((x-tx)*(x-tx)+(y-ty)*(y-ty));
		float nowSpd = battery_.sx()*battery_.sx()+battery_.sy()*battery_.sy();
		float useGauge = nearFact * (s - nowSpd);
		if (useGauge < 0.2) useGauge = 0.2;

		if (useGauge > game_.player(color_).gauge()) return;
		game_.player(color_).useGauge(useGauge);

		float a = atan2(battery_.sx(), battery_.sy());
		battery_.setSX(sin(a)*s);
		battery_.setSY(cos(a)*s);
	}
	void doChangeDirectionAbs(float a0) {
		float a = convRelAngle(a0);

		float px = game_.player(color_).x();
		float py = game_.player(color_).y();
		float tx = game_.player(Colors.opposite(color_)).x();
		float ty = game_.player(Colors.opposite(color_)).y();
		float x = battery_.x();
		float y = battery_.y();
		float nearFact =
			((tx-px)*(tx-px)+(ty-py)*(ty-py))/((x-tx)*(x-tx)+(y-ty)*(y-ty));

		float nowAng;
		if (color_ == Colors.WHITE)
			nowAng = atan2(battery_.sx(), battery_.sy());
		else
			nowAng = atan2(battery_.sx(), -battery_.sy());

		float useGauge = nearFact * (fabs(nowAng) - fabs(a));
		if (useGauge < 0.2) useGauge = 0.2;

		if (useGauge > game_.player(color_).gauge()) return;
		game_.player(color_).useGauge(useGauge);

		float s = sqrt(battery_.sx()*battery_.sx()+battery_.sy()*battery_.sy());
		battery_.setSX(sin(a)*s);
		float sy = cos(a)*s;
		if (color_ == Colors.BLACK) sy = -sy;
		battery_.setSY(sy);
		battery_.noticeChangeDirection();
	}
	void doChangeDirectionAim(float a0) {
		float a = convRelAngle(a0);

		float px = game_.player(color_).x();
		float py = game_.player(color_).y();
		float tx = game_.player(Colors.opposite(color_)).x();
		float ty = game_.player(Colors.opposite(color_)).y();
		float x = battery_.x();
		float y = battery_.y();
		float nearFact =
			((tx-px)*(tx-px)+(ty-py)*(ty-py))/((x-tx)*(x-tx)+(y-ty)*(y-ty));

		float nowAngSpeed = atan2(battery_.sx(), battery_.sy());
		float nowAngTarget = atan2(tx-battery_.x(), ty-battery_.y());
		float nowAng = fabs(nowAngSpeed - nowAngTarget);
		if (nowAng > PI) nowAng = PI * 2 - nowAng;
		float useGauge = nearFact * (nowAng - fabs(a)) * 1.2;
		if (useGauge < 0.2) useGauge = 0.2;
		if (useGauge > game_.player(color_).gauge()) return;
		game_.player(color_).useGauge(useGauge);

		float s = sqrt(battery_.sx()*battery_.sx()+battery_.sy()*battery_.sy());
		a += nowAngTarget;
		battery_.setSX(sin(a)*s);
		battery_.setSY(cos(a)*s);
		battery_.noticeChangeDirection();
	}

private:
	float convRelAngle(float a) {
		float r = a;
		while (r < -PI) r += PI * 2;
		while (r > PI) r -= PI * 2;
		return r;
	}

protected:
	lua_State lua_;

	Game game_;
	Charactor battery_;

	char[] func_;

	int color_;


}
