import std.string;
import std.math;

import SDL_gfxPrimitives;

import lua;

import cave;
import player;
import shot;
import log;
import mysdl;
import sprite;
import bullet;
import cpuinput;

class Game {
public:
	this() {
		screen_ = Cave.obj().screen();

		player_[0] = new Player(this, Cave.obj().padInput(), 0);
        anotherPlayerInput_ = PadInput.generate("save/key2.conf");
		player_[1] = new Player(this, null, 1);
		cpu_ = new CpuInput(player_[1]);
        if (Cave.rc.getInt(`playerModeIsCpu`)) {
            player_[1].setInput(cpu_);
        }
        else {
            player_[1].setInput(anotherPlayerInput_);
        }

		for (int c = 0; c < 2; c++) {
			SDL_Surface*[] surfs;
			static int x[] = [0, 0, 0, -1, -1, -1, 0, 0];
			for (int i = 0; i < 1; i++) {
				SDL_Surface* s = MySDL_CreateRGBSurface(19, 19);
				int trans = SDL_MapRGB(s.format, 0,255,0);
				SDL_FillRect(s, cast(SDL_Rect*)0, trans);
				filledCircleColor(s, 9, 9, 5, Colors.shotCore2[c]);
				circleColor(s, 9, 9, 9, Colors.shot[c]);
				SDL_SetColorKey(s, SDL_SRCCOLORKEY|SDL_RLEACCEL, trans);
				surfs ~= s;
			}
			sprites_[c] ~= new Sprite(surfs);
		}

		static Sint16 x[] = [ 0, -1, 1, -2, 2, 1, -2, 0];
		static Sint16 ys[] = [ 0, 1, 1, 3, 3, 3, 3, 3];
		static Sint16 ye[] = [ 21, 20, 20, 18, 18, 18, 18, 18];
		for (int c = 0; c < 2; c++) {
			Uint32 cols[];
			cols ~= Colors.shotCore[c];
			cols ~= Colors.shotCore[c];
			cols ~= Colors.shotCore[c];
			cols ~= Colors.shotCorePale[c];
			cols ~= Colors.shotCorePale[c];
			cols ~= Colors.shotPale[c];
			cols ~= Colors.shotPale[c];
			cols ~= Colors.shot[c];
			SDL_Surface*[] surfs;
			for (int i = 0; i < 32; i++) {
				SDL_Surface* s = MySDL_CreateRGBSurface(23, 23);
				int trans = SDL_MapRGB(s.format, 0,255,0);
				SDL_FillRect(s, cast(SDL_Rect*)0, trans);

				for (int j = 0; j < x.length; j++) {
					float r = PI / 16 * i;
					int rx = x[j];
					int rys = ys[j] - 12;
					int rye = ye[j] - 12;
					int x1 = cast(int)(cos(r) * rx + sin(r) * rys + 12);
					int y1 = cast(int)(-sin(r) * rx + cos(r) * rys + 12);
					int x2 = cast(int)(cos(r) * rx + sin(r) * rye + 12);
					int y2 = cast(int)(-sin(r) * rx + cos(r) * rye + 12);
					lineColor(s, x1, y1, x2, y2, cols[j]);
				}

				SDL_SetColorKey(s, SDL_SRCCOLORKEY|SDL_RLEACCEL, trans);

				surfs ~= s;
			}
			sprites_[c] ~= new Sprite(surfs);
		}

		for (int j = 0; j < 2; j++) {
			for (int i = 0; i < shots_[j].length; i++) {
				shots_[j][i] = null;
			}
			shotNum_[j] = 0;
		}

		turn_ = 0;
	}

	bit isEnd() {
		return isEnd_;
	}

	void run() {
		cacheShot();
		runPlayer();
		runShot();
		procDead();
		draw();

		turn_++;
	}

	void procDead() {
		for (int j = 0; j < 2; j++) {
			for (int i = 0; i < shots_[j].length; i++) {
				if (shots_[j][i] && !shots_[j][i].alive()) {
					delete shots_[j][i];
					shots_[j][i] = null;
				}
			}
		}
	}

	void runPlayer() {
		player_[0].move();
		player_[1].move();
	}

	void cacheShot() {
		for (int j = 0; j < 2; j++) {
			int n = 0;
			for (int i = 0; i < shots_[j].length; i++) {
				if (!shots_[j][i]) continue;
				Shot s = shots_[j][i];
				shotsX_[j][n] = s.x();
				shotsY_[j][n] = s.y();
				shotsSX_[j][n] = s.sx();
				shotsSY_[j][n] = s.sy();
				n++;
			}
		}
	}

	void runShot() {
		for (int j = 0; j < 2; j++) {
			int opp = Colors.opposite(j);
			float px = player_[opp].x();
			float py = player_[opp].y();
			for (int i = 0; i < shots_[j].length; i++) {
				if (!shots_[j][i]) continue;
				Shot s = shots_[j][i];
				s.move();
				s = shots_[j][i];
				float x = s.x();
				float y = s.y();
				if (s && (x < -10 || y < -10 ||
						  x > screen_.w+10 || y > screen_.h+10))
				{
					shots_[j][i].setAlive(false);
					shotNum_[j]--;
				}

				float size = 3;
				float sspd = fabs(s.sx()) + fabs(s.sy());
				if (sspd > 3.5) {
					size = 5;
				}
				else if (sspd > 1.5) {
					size = sspd + 1.5;
				}
				if (fabs(px - x) < size && fabs(py - y) < size) {
					player_[opp].damage();
				}
			}
		}
	}

	void draw() {
		if ((Cave.obj.turn & 3) == 0) {
			player_[0].drawField(screen_, 0);
			player_[1].drawField(screen_, 1);
		}

		for (int i = 0; i < shots_[0].length; i++) {
			if (!shots_[0][i]) continue;
			if (shots_[0][i].y() < (screen_.h>>1)) continue;
			shots_[0][i].draw(screen_, 0);
		}
		for (int i = 0; i < shots_[1].length; i++) {
			if (!shots_[1][i]) continue;
			shots_[1][i].draw(screen_, 1);
		}
		for (int i = 0; i < shots_[0].length; i++) {
			if (!shots_[0][i]) continue;
			if (shots_[0][i].y() >= (screen_.h>>1)) continue;
			shots_[0][i].draw(screen_, 0);
		}

		player_[0].drawCore(screen_, 0);
		player_[1].drawCore(screen_, 1);
	}

private:
	bit procCost(float x, float y, float d, float s, int col, bit aim) {
		float spdFact = (s < 1.5) ? 1 : (s - 0.5) / 2;
		float angFact = (1 - fabs(d) * 2);
		if (aim) angFact *= 1.2;
		if (angFact < 0.2) angFact = 0.2;
		static float simFact = 1;
		if (player_[col].lastBulletTurn() == turn()) {
			simFact *= 1.1;
		}
		else {
			simFact = 1;
		}
		float useGauge = spdFact * angFact * simFact;
		float px = player_[col].x();
		float py = player_[col].y();
		if (px != x || py != y) {
			float tx = player_[Colors.opposite(col)].x();
			float ty = player_[Colors.opposite(col)].y();
			float nearFact =
				((tx-px)*(tx-px)+(ty-py)*(ty-py))/((x-tx)*(x-tx)+(y-ty)*(y-ty));
			if (nearFact < 0.2) nearFact = 0.2;
			useGauge *= nearFact;
		}
		if (useGauge > player_[col].gauge()) {
			return false;
		}

		player_[col].useGauge(useGauge);
		player_[col].setLastBulletTurn(turn());

		return true;
	}

	void getSpeedAim(float x, float y, float d, float s, int col,
					 out float sx, out float sy) {
		int opp = Colors.opposite(col);
		float dx = player_[opp].x() - x;
		float dy = player_[opp].y() - y;
		float ang = atan2(dy, dx) + d;
		sx = cos(ang) * s;
		sy = sin(ang) * s;
	}

public:
	void addShotAim(float x, float y, float d, float s,
					int col, int type)
	in {
		if (type >= sprites_.length) {
			Log.error("bullet type " ~ std.string.toString(type) ~ " is invalid");
		}
	}
	body {
		for (int i = 0; i < shots_[col].length; i++) {
			if (!shots_[col][i]) {
				if (!procCost(x, y, d, s, col, true)) return;

				float sx, sy;
				getSpeedAim(x, y, d, s, col, sx, sy);

				shots_[col][i] = new Shot(x, y, sx, sy, sprites_[col][type]);
				shotNum_[col]++;

				cpu_.registerShot(x, y, sx, sy);

				return;
			}
		}
	}

	void addShotAbs(float x, float y, float d, float s,
					int col, int type)
	in {
		if (type >= sprites_.length) {
			Log.error("bullet type " ~ std.string.toString(type) ~ " is invalid");
		}
	}
	body {
		for (int i = 0; i < shots_[col].length; i++) {
			if (!shots_[col][i]) {
				if (!procCost(x, y, d, s, col, false)) return;

				float sx = sin(d) * s;
				float sy = cos(d) * s;
				if (col == Colors.BLACK) sy = -sy;

				shots_[col][i] = new Shot(x, y, sx, sy, sprites_[col][type]);
				shotNum_[col]++;

				cpu_.registerShot(x, y, sx, sy);

				return;
			}
		}
	}

	void addBulletAim(float x, float y, float d, float s,
					   int col, int type, lua_State lua, char[] fn)
	in {
		if (type >= sprites_.length) {
			Log.error("bullet type " ~ std.string.toString(type) ~ " is invalid");
		}
	}
	body {
		for (int i = 0; i < shots_[col].length; i++) {
			if (!shots_[col][i]) {
				if (!procCost(x, y, d, s, col, true)) return;

				float sx, sy;
				getSpeedAim(x, y, d, s, col, sx, sy);

				shots_[col][i] = new Bullet(x, y, sx, sy, sprites_[col][type],
											lua, fn, this, col);
				shotNum_[col]++;

				cpu_.registerShot(x, y, sx, sy);

				return;
			}
		}
	}

	void addBulletAbs(float x, float y, float d, float s,
					  int col, int type, lua_State lua, char[] fn)
	in {
		if (type >= sprites_.length) {
			Log.error("bullet type " ~ std.string.toString(type) ~ " is invalid");
		}
	}
	body {
		for (int i = 0; i < shots_[col].length; i++) {
			if (!shots_[col][i]) {
				if (!procCost(x, y, d, s, col, false)) return;

				float sx = sin(d) * s;
				float sy = cos(d) * s;
				if (col == Colors.BLACK) sy = -sy;

				shots_[col][i] = new Bullet(x, y, sx, sy, sprites_[col][type],
											lua, fn, this, col);
				shotNum_[col]++;

				cpu_.registerShot(x, y, sx, sy);

				return;
			}
		}
	}

	void vanishBullet(Charactor bullet, int col) {
		for (int i = 0; i < shots_[col].length; i++) {
			if (shots_[col][i] && shots_[col][i] == bullet) {
				shots_[col][i].setAlive(false);
				shotNum_[col]--;
				return;
			}
		}
		assert(false);
	}

	Player player(int color) {
		return player_[color];
	}

	int turn() { return turn_; }

	void getPlayerBullets(int col, out int len, out float* x, out float* y,
						 out float* sx, out float* sy) {
		len = shotNum_[col];
		x = shotsX_[col];
		y = shotsY_[col];
		sx = shotsSX_[col];
		sy = shotsSY_[col];
	}

private:
	SDL_Surface* screen_;

	bit isEnd_;

	Player[2] player_;

	static const int SHOT_NUM = 500;
	Shot[SHOT_NUM][2] shots_;
	int shotNum_[2];

	float[SHOT_NUM][2] shotsX_;
	float[SHOT_NUM][2] shotsY_;
	float[SHOT_NUM][2] shotsSX_;
	float[SHOT_NUM][2] shotsSY_;

	Sprite[][2] sprites_;

	int turn_;

    PadInput anotherPlayerInput_;
	CpuInput cpu_;

};
